from __future__ import with_statement

import os
import sys

import functools
import unittest
import logging

import pickle
import shutil
import tempfile

import jsonpickle
import jsonpickle.pickler
import jsonpickle.unpickler

import currypy as CurryModule

def foo(a, b=3):
    return a*b 


class TestPickle(unittest.TestCase):
    """
    """

    def setUp(self):

        self.rootDir = tempfile.mkdtemp()
        self._path = os.path.join(self.rootDir, 'foo')
        return
    
    def tearDown(self):
        if os.path.exists(self.rootDir):
            shutil.rmtree(self.rootDir)
        
        return
    

    def testCurryFromPartialObject(self):

        partial = functools.partial(foo, 2, b=5)

        curriedObj = CurryModule.Curry.curryFromPartialObject(partial)
        curriedPartial = curriedObj._partial

        self.assertEquals(curriedPartial.func, partial.func)
        self.assertEquals(curriedPartial.args, partial.args)
        self.assertEquals(curriedPartial.keywords, partial.keywords)

        partial = functools.partial(foo)
        curriedObj = CurryModule.Curry.curryFromPartialObject(partial)
        curriedPartial = curriedObj._partial

        self.assertEquals(curriedPartial.func, partial.func)
        self.assertEquals(curriedPartial.args, ())
        self.assertEquals(curriedPartial.keywords, {})

        return


    def testEquals(self):

        partial = functools.partial(foo, 2, b=5)
        curryObj1 = CurryModule.Curry.curryFromPartialObject(partial)
        curryObj2 = CurryModule.Curry.curryFromPartialObject(partial)
        
        self.assertFalse(curryObj1._partial is curryObj2._partial)
        self.assertEquals(curryObj1, curryObj2)
        
        return

    
    def testPickle1(self):

        curriedObj = CurryModule.Curry(foo, 2, b=5)
        dumped = False
        with open(self._path, 'w') as f:
            pickle.dump(curriedObj, f)
            dumped =True
        assert dumped

        loaded = False
        with open(self._path, 'r') as f:
            curried = pickle.load(f)
            assert curried() is 10
            loaded = True
        assert loaded

        return

    def testPickle2(self):

        curriedObj = CurryModule.Curry(foo, 2)
        dumped = False
        with open(self._path, 'w') as f:
            pickle.dump(curriedObj, f)
            dumped =True
        assert dumped

        loaded = False
        with open(self._path, 'r') as f:
            curried = pickle.load(f)
            assert curried(b=7) is 14
            loaded = True
        assert loaded

        return


    def testPickle3(self):

        curriedObj = CurryModule.Curry(foo, b=5)
        dumped = False
        with open(self._path, 'w') as f:
            pickle.dump(curriedObj, f)
            dumped =True
        assert dumped

        loaded = False
        with open(self._path, 'r') as f:
            curried = pickle.load(f)
            assert curried(7) is 35
            loaded = True
        assert loaded

        return

        
    # END class TestPickle
    pass


class TestJsonPickle(unittest.TestCase):
    """
    """

    def setUp(self):

        self.rootDir = tempfile.mkdtemp()
        self._path = os.path.join(self.rootDir, 'foo')
        return
    
    def tearDown(self):
        if os.path.exists(self.rootDir):
            shutil.rmtree(self.rootDir)
        
        return

    
    def testJsonPickle1(self):

        pickler = jsonpickle.Pickler()

        curriedObj = CurryModule.Curry(foo, 2, b=5)

        jsonVal = pickler.flatten(curriedObj)

        unpickler = jsonpickle.unpickler.Unpickler()
        newCurriedObj = unpickler.restore(jsonVal)

        self.assertEquals(10, newCurriedObj())

        return

    def testJsonPickle2(self):

        pickler = jsonpickle.Pickler()

        curriedObj = CurryModule.Curry(foo, 2, b=5)

        jsonVal = pickler.flatten(curriedObj)
        dumped = False
        with open(self._path, 'w') as f:
            pickle.dump(jsonVal, f)
            dumped = True
            pass

        loaded = False
        with open(self._path, 'r') as f:
            v = pickle.load(f)
            unpickler = jsonpickle.unpickler.Unpickler()
            newCurriedObj = unpickler.restore(v)
            loaded = True

        self.assertEquals(10, newCurriedObj())

        return

    # END class JsonPickle
    pass

def main():

    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestCase, 'test'))
    
    runner = unittest.TextTestRunner()
    runner.run(suite)
    return

if __name__=="__main__":
    main()




from setuptools import setup, find_packages

setup(
    name='currypy',
    version='0.1.1',
    packages=find_packages(
        'src',exclude=["*.test", "*.test.*", "test.*", "test"]),
    package_dir={'':'src'},
    test_suite="test",

    # metadata for upload to PyPI
    author = "michael j pan",
    author_email = "mikepan@gmail.com",
    description = "pickleable partial functions",
    license = "New BSD",
    keywords = "functools partial curry pickle",
    url = "http://code.google.com/p/currypy/",
    long_description = "functools.partial allows for the currying of arguments. However, being implemented in C, functools.partial objects are not pickleable. This package implements a wrapper class for the partial objects, such that enables pickling of the objects.",
    platforms = ["All"]

)



import functools

import pickle


class Curry(object):

    @staticmethod
    def curryFromPartialObject(partial):
        args = partial.args or ()
        kwds = partial.keywords or {}
        return Curry(partial.func, *args, **kwds)

    def __init__(self, function, *args, **kwds):
        self.initPartialObject(function, *args, **kwds)
        return

    def __eq__(self, other):
        if self is other:
            return True

        if not isinstance(other, self.__class__):
            return False

        selfPartial = self._partial
        otherPartial = other._partial
        if selfPartial.func is not otherPartial.func:
            return False
        if not selfPartial.args == otherPartial.args:
            return False
        if not selfPartial.keywords == otherPartial.keywords:
            return False
        return True


    def __call__(self, *args, **kwds):
        return self._partial(*args, **kwds)

    def initPartialObject(self, function, *args, **kwds):
        self._partial = functools.partial(function,
                                          *args,
                                          **kwds)
        return

    def __getstate__(self):
        odict = self.__dict__.copy()
        
        odict['function'] = pickle.dumps(self._partial.func)
        odict['args'] = self._partial.args
        odict['keywords'] = self._partial.keywords
        del odict['_partial']
        return odict


    def __setstate__(self, dict):

        function = pickle.loads(dict['function'])
        args = dict['args']
        keywords = dict['keywords']

        self.initPartialObject(function, *args, **keywords)

        return

    # END class Curry
    pass
